@@ -509,7 +509,7 @@ srl_read_varint_uv_offset(pTHX_ srl_decoder_t *dec, const char * const errstr)
 
     if (dec->buf_start + len >= dec->pos) {
         ERRORf4("Corrupted packet%s. Offset %lu points past current position %lu in packet with length of %lu bytes long",
-                errstr, len, BUF_POS_OFS(dec), dec->buf_len);
+                errstr, (unsigned long)len, (unsigned long)BUF_POS_OFS(dec), (unsigned long)dec->buf_len);
     }
     return len;
 }
@@ -549,7 +549,7 @@ srl_fetch_item(pTHX_ srl_decoder_t *dec, UV item, const char const *tag_name)
 {
     SV *sv= (SV *)PTABLE_fetch(dec->ref_seenhash, (void *)item);
     if (expect_false( !sv ))
-        ERRORf2("%s(%d) references an unknown item", tag_name, item);
+        ERRORf2("%s(%d) references an unknown item", tag_name, (int)item);
     return sv;
 }
 
@@ -702,7 +702,7 @@ srl_read_hash(pTHX_ srl_decoder_t *dec, SV* into, U8 tag) {
     /* Limit the maximum number of hash keys that we accept to whetever was configured */
     if (dec->max_num_hash_entries != 0 && num_keys > dec->max_num_hash_entries) {
         ERRORf2("Got input hash with %u entries, but the configured maximum is just %u",
-                num_keys, dec->max_num_hash_entries);
+                (int)num_keys, (int)dec->max_num_hash_entries);
     }
 
     ASSERT_BUF_SPACE(dec,num_keys*2,"while reading hash contents, insuffienct remaining tags for number of keys specified");
@@ -871,7 +871,7 @@ srl_read_objectv(pTHX_ srl_decoder_t *dec, SV* into)
         ERROR("Corrupted packet. OBJECTV used without preceding OBJECT to define classname");
     av= (AV *)PTABLE_fetch(dec->ref_bless_av, (void *)ofs);
     if (NULL == av) {
-        ERRORf1("Corrupted packet. OBJECTV references unknown classname offset: %lu", ofs);
+        ERRORf1("Corrupted packet. OBJECTV references unknown classname offset: %lu", (unsigned long)ofs);
     }
     /* now deparse the thing we are going to bless */
     srl_read_single_value(aTHX_ dec, into);
@@ -973,7 +973,7 @@ srl_read_object(pTHX_ srl_decoder_t *dec, SV* into)
         sv_2mortal((SV*)av);
         PTABLE_store(dec->ref_bless_av, (void *)storepos, (void *)av);
     } else if (NULL == (av= (AV *)PTABLE_fetch(dec->ref_bless_av, (void *)storepos)) ) {
-        ERRORf1("Panic, no ref_bless_av for %lu",storepos);
+        ERRORf1("Panic, no ref_bless_av for %lu", (unsigned long)storepos);
     }
 
     /* we now have a stash so we /could/ bless... except that
@@ -1003,8 +1003,15 @@ srl_read_reserved(pTHX_ srl_decoder_t *dec, U8 tag, SV* into)
     sv_setsv(into, &PL_sv_undef);
 }
 
-#ifdef SvRX
-#define MODERN_REGEXP
+#if ((PERL_VERSION > 10) || (PERL_VERSION == 10 && PERL_SUBVERSION > 0 ))
+#	define MODERN_REGEXP
+#	define REGEXP_HAS_P_MODIFIER
+#elif PERL_VERSION==10
+#	define TRANSITION_REGEXP
+#	define REGEXP_HAS_P_MODIFIER
+#else
+#	warning "OLD REGEXP"
+#	define OLD_REGEXP
 #endif
 
 static SRL_INLINE void
@@ -1035,7 +1042,7 @@ srl_read_regexp(pTHX_ srl_decoder_t *dec, SV* into)
                 case 'x':
                     flags= flags | PMf_EXTENDED;
                     break;
-#ifdef MODERN_REGEXP
+#ifdef REGEXP_HAS_P_MODIFIER
                 case 'p':
                     flags = flags | PMf_KEEPCOPY;
                     break;
@@ -1045,7 +1052,7 @@ srl_read_regexp(pTHX_ srl_decoder_t *dec, SV* into)
                     break;
             }
         }
-#ifdef SvRX
+#ifdef MODERN_REGEXP
         {
             /* This is ugly. We have to swap out the insides of our SV
              * with the one we get back from CALLREGCOMP, as there is no
@@ -1076,9 +1083,17 @@ srl_read_regexp(pTHX_ srl_decoder_t *dec, SV* into)
 
             if (SWAP_DEBUG) { warn("after swap:"); sv_dump(into); sv_dump(referent); }
 
+            SvREFCNT_dec(sv_pat); /* I think we need this or we leak */
             /* and now throw away the head we got from the regexp engine. */
             SvREFCNT_dec(referent);
         }
+#elif defined( TRANSITION_REGEXP )
+        {
+            REGEXP *referent = CALLREGCOMP(aTHX_ sv_pat, flags);
+            SvREFCNT_dec(sv_pat);
+            sv_magic( into, (SV*)referent, PERL_MAGIC_qr, 0, 0);
+            SvFLAGS( into ) |= SVs_SMG;
+        }
 #else
         {
             PMOP pm; /* grr */
@@ -1128,10 +1143,10 @@ srl_read_copy(pTHX_ srl_decoder_t *dec, SV* into)
 {
     UV item= srl_read_varint_uv_offset(aTHX_ dec, " while reading COPY tag");
     if (expect_false( dec->save_pos )) {
-        ERRORf1("COPY(%d) called during parse", item);
+        ERRORf1("COPY(%d) called during parse", (int)item);
     }
     if (expect_false( (IV)item > dec->buf_end - dec->buf_start )) {
-        ERRORf1("COPY(%d) points out of packet",item);
+        ERRORf1("COPY(%d) points out of packet", (int)item);
     }
     dec->save_pos= dec->pos;
     dec->pos= dec->buf_start + item;
